#import "Controller.h"
#import "Cell.h"

@implementation Controller

-(id)init {
	if (self = [super init]) {
		depth = 2;
		N = 5; NN = 25; NNN = 125;
	}
	return self;
}

-(void)awakeFromNib {
	[drawerController open];
	[cosmosView setMagnification:1.0];
	[cosmosView setRotationSpeed:0.5];
	[cosmosView setSubcells:FALSE];
	[self start:nil];
}

-(IBAction)start:(id)sender {
	[cosmosView clearObjects];
	[self iterateWithCount:depth x:0.0 y:0.0 z:0.0 length:1.0];
}

-(IBAction)setIterationDepth:(id)slider {
	if (depth != [slider intValue]) {
		depth = [slider intValue];
		[self start:nil];
	}
}

-(IBAction)setRotationSpeed:(id)slider {
	[cosmosView setRotationSpeed:[slider floatValue]];
	[cosmosView setNeedsDisplay:YES];
}

-(IBAction)setMagnification:(id)slider {
	[cosmosView setMagnification:[slider floatValue]];
	[cosmosView setNeedsDisplay:YES];
}

-(IBAction)setN:(id)slider {
	if (N != [slider intValue]) {
		N = [slider intValue];
		NN = N*N;
		NNN = N*NN;
		[self start:nil];
	}
}

-(IBAction)setShape:(id)sender {
	shape = [sender selectedRow];
	[cosmosView setShape:shape];
	[cosmosView setNeedsDisplay:YES];
}

-(IBAction)setSubcells:(id)sender {
	subcells = ([sender state] == NSOffState) ? FALSE : TRUE;
	[cosmosView setSubcells:subcells];
	[cosmosView setNeedsDisplay:YES];
}	

-(IBAction)setAnimation:(id)sender {
	[cosmosView setAnimation:([sender state] == NSOnState)];
	[cosmosView setNeedsDisplay:YES];
}

-(void)iterateWithCount:(int)count
					  x:(double)x
					  y:(double)y
					  z:(double)z
				 length:(double)length {
	
	int a[N], used[NNN], i, j, k=0;
	int ix,iy,iz;
	Cell *cell;
		
	cell = [[Cell alloc] init];

	[cell setX:x];
	[cell setY:y];
	[cell setZ:z];
	[cell setRadius:length];
	[cell setDepth:count];
	[cosmosView addCell:cell];
	[cell release];

	[cosmosView setNeedsDisplay:YES];
	if (count==0) return;
		
	for (i=0; i<NNN; i++) used[i] = 0;
		
	for (j=0; j<N; j++) {					// j iterates over each 2-d slice of the cube subcell
		do {
			k = (int) (rand() % NNN);		// pick a random kth subcell in NxNxN cube
		} while (used[k]);					// leave loop if not previously chosen (almost always true)
		used[k] = 1;						// mark this subcell as used
		a[j] = k;							// the jth 2-d slice is used to calculate (below) the next subcell
	}
	count--;
	length = length / ((double)N);
		
	for (i=0; i<N; i++) {
									//									example : a[i] = 82, N = 5
		ix = a[i] / NN;				// 0-N value						ix = (int)(83/(int)25) = 3
		iy = (a[i] - NN * ix) / N;  // the remainder value (0-N)		iy = (int)((82-3*25)/5) = 1
		iz = a[i] % N;				// modulus (0-N)					iz = 82 mod 5 = 2
		
		[self iterateWithCount:count
							 x:(x + (length*(float)ix))
							 y:(y + (length*(float)iy))
							 z:(z + (length*(float)iz))
						length:length];
	}
	[cosmosView setNeedsDisplay:YES];
}

@end
