#import "HJ3dCage.h"
#import "math.h"

@implementation HJ3dCage

#pragma mark --- cell management ----

-(void)addCell:(Cell *)newCell {
	if (objects == nil) {
		objects = [[NSMutableArray alloc] init];
	}
	[objects addObject:newCell];
	stateChanged = TRUE;
}

-(void)setMagnification:(float)newMagnification {
	magnification = newMagnification;
	stateChanged = TRUE;
}

-(void)setRotationSpeed:(float)newRotationSpeed {
	rotationSpeed = newRotationSpeed;
	stateChanged = TRUE;
}

-(void)setShape:(int)newShape {
	shape = newShape;
	stateChanged = TRUE;
}

-(void)setSubcells:(BOOL)newSubcells {
	subcells = newSubcells;
	stateChanged = TRUE;
}

-(void)clearObjects {
	[objects removeAllObjects];
	stateChanged = TRUE;
}

-(void)setAnimation:(BOOL)animationState {
	animate = animationState;
}

#pragma mark ---- interface methods ----

-(void)scrollWheel:(NSEvent *)e {
	float delta = [e deltaX] + [e deltaY] + [e deltaZ];
	if (delta) {
		yHeight += delta / 2.0;
		stateChanged = TRUE;
	}
}

-(void)mouseDown:(NSEvent *)mouse {
    startPosition = [self convertPoint:[mouse locationInWindow] fromView:[self superview]];
}

-(void)mouseDragged:(NSEvent *)mouse {
    currentPosition = [self convertPoint:[mouse locationInWindow] fromView:[self superview]];

	xzAngle -= (currentPosition.y - startPosition.y) * 0.001;
	cameraAngle -= (currentPosition.x - startPosition.x) * 0.001 * 3.14159 / 180.0;

	startPosition = currentPosition;
	stateChanged = TRUE;
}


#pragma mark ---- drawing methods ----

-(void)awakeFromNib {
	
	NSOpenGLPixelFormatAttribute attr[] = {
		NSOpenGLPFADoubleBuffer, NSOpenGLPFAAccelerated,
		NSOpenGLPFAColorSize, BITS_PER_PIXEL,
		NSOpenGLPFADepthSize, DEPTH_SIZE, 0
	};
	
	nsglFormat = [[NSOpenGLPixelFormat alloc] initWithAttributes:attr];
	[self setPixelFormat:nsglFormat];
	[[self openGLContext] makeCurrentContext];
	[self initGL];

	cameraAngle = 0.0;
	magnification = 1.0;
	rotationSpeed = 2.0;
	yHeight = 0.0;
	boxWidth = BOXWIDTH;
	boxVisible = TRUE;
	animate = TRUE;
	stateChanged = TRUE;
	subcells = FALSE;
	
	animationInterval = DEFAULT_ANIMATION_INTERVAL;
	animationTimer = [[NSTimer scheduledTimerWithTimeInterval:animationInterval
													   target:self
													 selector:@selector(drawRect)
													 userInfo:nil
													  repeats:YES] retain];
	[[NSRunLoop currentRunLoop] addTimer:animationTimer
								 forMode:NSEventTrackingRunLoopMode];
	[[NSRunLoop currentRunLoop] addTimer:animationTimer
								 forMode:NSModalPanelRunLoopMode];
	[self setNeedsDisplay:TRUE];
}

-(void)drawOutlineBox {
	if (boxVisible) {
		glBegin(GLU_LINE);
		{
			glColor4f( 0.0, 1.0, 0.0, 0.2 );
			glutWireCube( BOXWIDTH+2.0 );
		}
		glEnd();
	}
}

-(void)drawObjects {
    GLfloat cubeColor[4]; //= { 0.3, 0.3, 0.8, 0.2 };
	int i;
	double x,y,z,size,sizeup;

	colorRotationAngle += 0.001;
	if (colorRotationAngle >= 360.0) colorRotationAngle = 0.0;
	cubeColor[0] = 0.75 + (0.25 * sin( colorRotationAngle) );		// .5 to 1 ranges
	cubeColor[1] = 0.75 + (0.25 * sin(colorRotationAngle * 2.0) );
	cubeColor[2] = 0.75 + (0.25 * sin(colorRotationAngle * 3.0) );
	cubeColor[3] = 0.2;
	
	glBegin(GLU_LINE);
	for (i=0; i<[objects count]; i++) {
		if (subcells || ([[objects objectAtIndex:i] depth] == 0)) {
			sizeup = SCALE * magnification;
			size = [[objects objectAtIndex:i] radius] * sizeup;

			x = ([[objects objectAtIndex:i] x] * sizeup) + (size / 2.0) - 0.5 * sizeup;
			y = ([[objects objectAtIndex:i] y] * sizeup) + (size / 2.0) - 0.5 * sizeup;
			z = ([[objects objectAtIndex:i] z] * sizeup) + (size / 2.0) - 0.5 * sizeup;
			
			if ([[objects objectAtIndex:i] depth] == 0)
				cubeColor[3] = 1.0;
			else
				cubeColor[3] = 0.2;
			
			glPushMatrix();
			glTranslatef( x, y, z );
			glMaterialfv( GL_FRONT, GL_DIFFUSE, cubeColor );
			if (shape == 1)
				glutSolidSphere( size, 8, 8 );
			else if (shape == 0)
				glutSolidCube( size );
			
			glPopMatrix();
		}
	}
	glEnd();
}

-(void)drawRect {
	if (animate || stateChanged) {
		glMatrixMode( GL_MODELVIEW );
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
		glEnable( GL_LIGHTING );
		
		if (subcells) {
			glEnable( GL_BLEND );
			glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
			glDisable( GL_DEPTH_TEST );
			glEnable( GL_CULL_FACE );
			//glEnable( GL_DEPTH_TEST );
			//glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
			//glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		} else {
			glDisable( GL_BLEND );
			glBlendFunc( GL_SRC_ALPHA, GL_ONE );
			glEnable( GL_DEPTH_TEST );
			//glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST );
			//glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
		}
		
		glLoadIdentity();
		[self positionCamera];
		[self drawOutlineBox];
		[self drawObjects];
		[[self openGLContext] flushBuffer];
		stateChanged = FALSE;
	}
}

-(void)initGL {
	float width = 10.0, height = 10.0;
	
	glClearColor(0.0, 0.0, 0.0, 0.5);   // set to black
	glClearDepth(5.0);					// default value
	
	glDepthFunc(GL_LESS);				// kind of depth testing
	glEnable(GL_DEPTH_TEST);			// enable depth testing
	glShadeModel(GL_SMOOTH);			// smooth color shading
	
	[self light1];
	
	glMatrixMode(GL_PROJECTION);
	{
		glLoadIdentity();
		gluPerspective(60.0,			// view angle of camera
					   (width/height),  // aspect ratio
					   0.1,				// near-plane cut-off
					   10000.0);		// far-plane cut-off
	}
	glMatrixMode(GL_MODELVIEW);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); // slight performance hit
}

-(void)light1 {
	GLfloat lightPosition[] = {20.0f, 0.0f, 0.0f, 1.0f };
	GLfloat lightAmbient[] =  { 0.5f, 0.5f, 1.0f, 1.0f };
	GLfloat lightDiffuse[] =  { 1.0f, 1.0f, 1.0f, 1.0f };

	/*
	float cameraRadians = cameraAngle * 180.0 / 3.14159;
	
	GLfloat lightPosition[] = {
		20.0 * sin( cameraRadians ),
		yHeight + BOXWIDTH * sin( xzAngle ),
		cos( cameraRadians ) * cos( xzAngle ),
		1.0f
	};
	*/
	glLightfv( GL_LIGHT1, GL_AMBIENT,  lightAmbient );
	glLightfv( GL_LIGHT1, GL_DIFFUSE,  lightDiffuse );
	glLightfv( GL_LIGHT1, GL_POSITION, lightPosition);
	glEnable(  GL_LIGHT1 );
	
	glColor4f( .5f, .7f, 1.0f, .5f ); // full brightness, 50% alpha
}
	
-(void)positionCamera {
	float cameraRadians;
	if (animate) {
		cameraAngle += (0.00005 * rotationSpeed);
	}
	if (cameraAngle >= 360.0) cameraAngle = 0.0;
	
	cameraRadians = cameraAngle * 180.0 / 3.14159;
	
	GLfloat cameraHome[3] = {
		BOXWIDTH * sin( cameraRadians ),
		yHeight + BOXWIDTH * sin( xzAngle ),
		BOXWIDTH * cos( cameraRadians ) * cos( xzAngle )
	};
	GLfloat centerHome[3] = { 0.0f, yHeight, 0.0f };
	
	gluLookAt(cameraHome[0], cameraHome[1], cameraHome[2],
			  centerHome[0], centerHome[1], centerHome[2],
			  0.0, 1.0, 0.0);
}

-(void)windowWillClose:(NSNotification *)notification {
	[animationTimer invalidate];
}

#pragma mark ---- set up/down methods ----

-(void)dealloc {
	[objects removeAllObjects];
	[super dealloc];
}

@end